#! /usr/bin/env python
# encoding: utf-8
# WARNING! Do not edit! https://waf.io/book/index.html#_obtaining_the_waf_file

from.import generic
import os.path as osp
from waflib.Utils import to_list
from waflib.Logs import debug,info,error,warn
mydir=osp.dirname(__file__)
package_descriptions=[('spdlog',dict(incs=['spdlog/spdlog.h'],libs=['spdlog'])),('ZLib',dict(incs=['zlib.h'],libs=['z'])),('FFTW',dict(incs=['fftw3.h'],libs=['fftw3f'],pcname='fftw3f')),('FFTWThreads',dict(libs=['fftw3f_threads'],pcname='fftw3f',mandatory=False)),('JsonCpp',dict(incs=["json/json.h"],libs=['jsoncpp'])),('Eigen',dict(incs=["Eigen/Dense"],pcname='eigen3')),('Jsonnet',dict(incs=["libjsonnet.h"],libs=['jsonnet'])),('TBB',dict(incs=["tbb/parallel_for.h"],libs=['tbb'],mandatory=False)),('HDF5',dict(incs=["hdf5.h"],libs=['hdf5'],mandatory=False)),('H5CPP',dict(incs=["h5cpp/all"],mandatory=False,extuses=('HDF5',))),('ZMQ',dict(incs=["zmq.h"],libs=['zmq'],pcname='libzmq',mandatory=False)),('CZMQ',dict(incs=["czmq.h"],libs=['czmq'],pcname='libczmq',mandatory=False)),('ZYRE',dict(incs=["zyre.h"],libs=['zyre'],pcname='libzyre',mandatory=False)),('ZIO',dict(incs=["zio/node.hpp"],libs=['zio'],pcname='libzio',mandatory=False,extuses=("ZYRE","CZMQ","ZMQ"))),]
def options(opt):
	opt.load('boost')
	opt.load('smplpkgs')
	opt.load('rootsys')
	opt.load('libtorch')
	opt.load('cuda')
	opt.load('kokkos')
	for name,desc in package_descriptions:
		generic._options(opt,name,desc.get("incs",None),desc.get("libs",None))
	opt.add_option('--build-debug',default='-O2 -ggdb3',help="Build with debug symbols")
def find_submodules(ctx):
	sms=list()
	for wb in ctx.path.ant_glob("**/wscript_build"):
		sms.append(wb.parent.name)
	sms.sort()
	return sms
def configure(cfg):
	print('Compile options: %s'%cfg.options.build_debug)
	cfg.load('boost')
	cfg.load('smplpkgs')
	for name,args in package_descriptions:
		generic._configure(cfg,name,**args)
	if getattr(cfg.options,"with_libtorch",False)is False:
		print("sans libtorch")
	else:
		cfg.load('libtorch')
	if getattr(cfg.options,"with_cuda",False)is False:
		print("sans CUDA")
	else:
		cfg.load('cuda')
	if getattr(cfg.options,"with_kokkos",False)is False:
		print("sans KOKKOS")
	else:
		cfg.load('kokkos')
	if getattr(cfg.options,"with_root",False)is False:
		print("sans ROOT")
	else:
		cfg.load('rootsys')
	def haveit(one):
		one=one.upper()
		if'LIB_'+one in cfg.env:
			cfg.env['HAVE_'+one]=1
			print('HAVE %s libs'%one)
		else:
			print('NO %s libs'%one)
	cfg.check_boost(lib='system filesystem graph thread program_options iostreams regex')
	haveit('boost')
	cfg.check(header_name="dlfcn.h",uselib_store='DYNAMO',lib=['dl'],mandatory=True)
	haveit('dynamo')
	cfg.check(features='cxx cxxprogram',lib=['pthread'],uselib_store='PTHREAD')
	haveit('pthread')
	cfg.env.CXXFLAGS+=to_list(cfg.options.build_debug)
	cfg.env.CXXFLAGS+=['-DEIGEN_FFTW_DEFAULT=1']
	cfg.env.LIB+=['z']
	submodules=find_submodules(cfg)
	for pkg,ext in[("root","HAVE_ROOTSYS"),("tbb","HAVE_TBB"),("tbb","LIB_FFTWTHREADS"),("cuda","HAVE_CUDA"),("hio","INCLUDES_H5CPP"),("pytorch","LIB_LIBTORCH"),("zio","LIB_ZIO LIB_ZYRE LIB_CZMQ LIB_ZMQ")]:
		exts=to_list(ext)
		for have in exts:
			if have in cfg.env or have in cfg.env.define_key:
				continue
			if pkg in submodules:
				print('Removing package "%s" due to lack of external dependency "%s"'%(pkg,have))
				submodules.remove(pkg)
	cfg.env.SUBDIRS=submodules
	info('Configured for submodules: %s'%(', '.join(submodules),))
	bch='WireCellUtil/BuildConfig.h'
	cfg.env['BuildConfig']=bch
	info('Writing build config: %s'%bch)
	cfg.write_config_header(bch)
def build(bld):
	bld.load('smplpkgs')
	bch=bld.path.find_or_declare(bld.env.BuildConfig)
	bld.install_files('${PREFIX}/include/'+bch.parent.name,bch)
	subdirs=bld.env.SUBDIRS
	print('Building: %s'%(', '.join(subdirs),))
	bld.recurse(subdirs)
	if hasattr(bld,"smplpkg_graph"):
		print("writing wct-depos.dot")
		bld.path.make_node("wct-deps.dot").write(str(bld.smplpkg_graph))
